import 'package:bones_api/bones_api.dart';
import 'package:reflection_factory/reflection_factory.dart';

import 'api_entities.dart';
import 'api_repositories.dart';

part 'module_account.reflection.g.dart';

@EnableReflection()
class ModuleAccount extends APIModule {
  ModuleAccount(APIRoot apiRoot) : super(apiRoot, 'account');

  final AddressAPIRepository addressRepository = AddressAPIRepository();

  final AccountAPIRepository accountRepository = AccountAPIRepository();

  @override
  void configure() {
    routes.anyFrom(reflection);
  }

  Future<APIResponse> auth(String? email, {String? password}) async {
    if (email == null) {
      return APIResponse.error(error: 'Invalid parameters!');
    }

    if (password == null) {
      return APIResponse.unauthorized();
    }

    var response = APIResponse.unauthorized();

    response.startMetric('db_select');

    var sel = await accountRepository.selectAccountByEmail(email);

    response.stopMetric('db_select');

    response.startMetric('after_sql');

    if (sel.isEmpty) {
      return response;
    }

    var account = sel.first;

    return account.checkPassword(password)
        ? response.asOk(payload: account.toJson()..remove('passwordHash'))
        : response;
  }

  APIResponse byState(String state) {
    var sel = accountRepository.selectAccountByAddressState(state);
    return APIResponse.ok(sel);
  }

  Future<APIResponse> add(
      {required String email,
      required String password,
      required String countryCode,
      required String state,
      required String city,
      String? address1,
      String? address2,
      String? zipCode}) async {
    var address = Address(countryCode, state, city, address1 ?? '',
        address2 ?? '', zipCode ?? '');

    var account = Account(email, password, address);

    var stored = await accountRepository.store(account);

    print('stored: $stored');

    return stored != null
        ? APIResponse.ok(account.toJson()..remove('passwordHash'))
        : APIResponse.error(error: "Can't store account");
  }

  Future<APIResponse> count() async {
    var count = await accountRepository.length();
    return APIResponse.ok(count);
  }

  Future<APIResponse> deleteByEmail(String email) async {
    var del = await accountRepository
        .deleteByQuery(' email == ? ', parameters: [email]);
    return APIResponse.ok(del);
  }
}
